@@ -1,6 +1,6 @@
 /* Copyright (c) 2002-2007 Jean-Marc Valin
    Copyright (c) 2008 CSIRO
-   Copyright (c) 2007-2012 Xiph.Org Foundation
+   Copyright (c) 2007-2013 Xiph.Org Foundation
    File: opusdec.c
 
    Redistribution and use in source and binary forms, with or without
@@ -35,12 +35,8 @@
 #if !defined WIN32 && !defined _WIN32
 #include <unistd.h>
 #endif
-#ifdef HAVE_GETOPT_H
+
 #include <getopt.h>
-#endif
-/*#ifndef HAVE_GETOPT_LONG
-#include "getopt_win.h"
-#endif*/
 #include <stdlib.h>
 #include <limits.h>
 #include <string.h>
@@ -288,7 +284,33 @@ FILE *out_file_open(char *outFile, int *wav_format, int rate, int mapping_family
    /*Open output file*/
    if (strlen(outFile)==0)
    {
-#if defined HAVE_SYS_SOUNDCARD_H
+#if defined HAVE_LIBSNDIO
+      struct sio_par par;
+
+      hdl = sio_open(NULL, SIO_PLAY, 0);
+      if (!hdl)
+      {
+         fprintf(stderr, "Cannot open sndio device\n");
+         quit(1);
+      }
+
+      sio_initpar(&par);
+      par.sig = 1;
+      par.bits = 16;
+      par.rate = rate;
+      par.pchan = *channels;
+
+      if (!sio_setpar(hdl, &par) || !sio_getpar(hdl, &par) ||
+        par.sig != 1 || par.bits != 16 || par.rate != rate) {
+          fprintf(stderr, "could not set sndio parameters\n");
+          quit(1);
+      }
+      *channels = par.pchan;
+      if (!sio_start(hdl)) {
+          fprintf(stderr, "could not start sndio\n");
+          quit(1);
+      }
+#elif defined HAVE_SYS_SOUNDCARD_H || defined HAVE_MACHINE_SOUNDCARD_H || HAVE_SOUNDCARD_H
       int audio_fd, format, stereo;
       audio_fd=open("/dev/dsp", O_WRONLY);
       if (audio_fd<0)
@@ -344,32 +366,6 @@ FILE *out_file_open(char *outFile, int *wav_format, int rate, int mapping_family
         perror("Cannot open output");
         quit(1);
       }
-#elif defined HAVE_LIBSNDIO
-      struct sio_par par;
-
-      hdl = sio_open(NULL, SIO_PLAY, 0);
-      if (!hdl)
-      {
-         fprintf(stderr, "Cannot open sndio device\n");
-         quit(1);
-      }
-
-      sio_initpar(&par);
-      par.sig = 1;
-      par.bits = 16;
-      par.rate = rate;
-      par.pchan = *channels;
-
-      if (!sio_setpar(hdl, &par) || !sio_getpar(hdl, &par) ||
-        par.sig != 1 || par.bits != 16 || par.rate != rate) {
-          fprintf(stderr, "could not set sndio parameters\n");
-          quit(1);
-      }
-      *channels = par.pchan;
-      if (!sio_start(hdl)) {
-          fprintf(stderr, "could not start sndio\n");
-          quit(1);
-      }
 #elif defined HAVE_SYS_AUDIOIO_H
       audio_info_t info;
       int audio_fd;
@@ -475,14 +471,13 @@ void usage(void)
 
 void version(void)
 {
-   printf("opusdec %s %s (using %s)\n",PACKAGE,VERSION,opus_get_version_string());
-   printf("Copyright (C) 2008-2012 Xiph.Org Foundation\n");
+   printf("opusdec %s %s (using %s)\n",PACKAGE_NAME,PACKAGE_VERSION,opus_get_version_string());
+   printf("Copyright (C) 2008-2013 Xiph.Org Foundation\n");
 }
 
 void version_short(void)
 {
-   printf("opusdec %s %s (using %s)\n",PACKAGE,VERSION,opus_get_version_string());
-   printf("Copyright (C) 2008-2012 Xiph.Org Foundation\n");
+   version();
 }
 
 /*Process an Opus header and setup the opus decoder based on it.
@@ -677,6 +672,7 @@ int main(int argc, char **argv)
    int preskip=0;
    int gran_offset=0;
    int has_opus_stream=0;
+   int has_tags_packet=0;
    ogg_int32_t opus_serialno;
    int dither=1;
    shapestate shapemem;
@@ -866,16 +862,32 @@ int main(int argc, char **argv)
             /*OggOpus streams are identified by a magic string in the initial
               stream header.*/
             if (op.b_o_s && op.bytes>=8 && !memcmp(op.packet, "OpusHead", 8)) {
+               if(has_opus_stream && has_tags_packet)
+               {
+                 /*If we're seeing another BOS OpusHead now it means
+                   the stream is chained without an EOS.*/
+                 has_opus_stream=0;
+                 if(st)opus_multistream_decoder_destroy(st);
+                 st=NULL;
+                 fprintf(stderr,"\nWarning: stream %" I64FORMAT " ended without EOS and a new stream began.\n",(long long)os.serialno);
+               }
                if(!has_opus_stream)
                {
+                 if(packet_count>0 && opus_serialno==os.serialno)
+                 {
+                   fprintf(stderr,"\nError: Apparent chaining without changing serial number (%" I64FORMAT "==%" I64FORMAT ").\n",
+                     (long long)opus_serialno,(long long)os.serialno);
+                   quit(1);
+                 }
                  opus_serialno = os.serialno;
                  has_opus_stream = 1;
+                 has_tags_packet = 0;
                  link_out = 0;
                  packet_count = 0;
                  eos = 0;
                  total_links++;
                } else {
-                 fprintf(stderr,"Warning: ignoring opus stream %" I64FORMAT "\n",(long long)os.serialno);
+                 fprintf(stderr,"\nWarning: ignoring opus stream %" I64FORMAT "\n",(long long)os.serialno);
                }
             }
             if (!has_opus_stream || os.serialno != opus_serialno)
@@ -914,7 +926,7 @@ int main(int argc, char **argv)
                  as described in the OggOpus spec.  But for commandline tools
                  like opusdec it can be desirable to exactly preserve the original
                  sampling rate and duration, so we have a resampler here.*/
-               if (rate != 48000)
+               if (rate != 48000 && resampler==NULL)
                {
                   int err;
                   resampler = speex_resampler_init(channels, 48000, rate, 5, &err);
@@ -927,6 +939,7 @@ int main(int argc, char **argv)
             {
                if (!quiet)
                   print_comments((char*)op.packet, op.bytes);
+               has_tags_packet=1;
                if(ogg_stream_packetout(&os, &op)!=0 || og.header[og.header_len-1]==255)
                {
                   fprintf(stderr, "Extra packets on initial tags page. Invalid stream.\n");
