@@ -1,6 +1,6 @@
 /* Copyright (C)2002-2011 Jean-Marc Valin
-   Copyright (C)2007-2012 Xiph.Org Foundation
-   Copyright (C)2008-2012 Gregory Maxwell
+   Copyright (C)2007-2013 Xiph.Org Foundation
+   Copyright (C)2008-2013 Gregory Maxwell
    File: opusenc.c
 
    Redistribution and use in source and binary forms, with or without
@@ -64,6 +64,7 @@
 #include <ogg/ogg.h>
 #include "wav_io.h"
 
+#include "picture.h"
 #include "opus_header.h"
 #include "opusenc.h"
 #include "diag_range.h"
@@ -79,7 +80,7 @@
 #endif
 
 static void comment_init(char **comments, int* length, const char *vendor_string);
-static void comment_add(char **comments, int* length, char *tag, char *val);
+static void comment_pad(char **comments, int* length, int amount);
 
 /*Write an Ogg page to a file pointer*/
 static inline int oe_write_page(ogg_page *page, FILE *fp)
@@ -96,21 +97,25 @@ static inline int oe_write_page(ogg_page *page, FILE *fp)
 
 void opustoolsversion(const char *opusversion)
 {
-  printf("opusenc %s %s (using %s)\n",PACKAGE,VERSION,opusversion);
-  printf("Copyright (C) 2008-2012 Xiph.Org Foundation\n");
+  printf("opusenc %s %s (using %s)\n",PACKAGE_NAME,PACKAGE_VERSION,opusversion);
+  printf("Copyright (C) 2008-2013 Xiph.Org Foundation\n");
 }
 
 void opustoolsversion_short(const char *opusversion)
 {
-  printf("opusenc %s %s (using %s)\n",PACKAGE,VERSION,opusversion);
-  printf("Copyright (C) 2008-2012 Xiph.Org Foundation\n");
+  opustoolsversion(opusversion);
 }
 
 void usage(void)
 {
   printf("Usage: opusenc [options] input_file output_file.opus\n");
   printf("\n");
-  printf("Encodes input_file using Opus. It can read the WAV, AIFF, or raw files.\n");
+  printf("Encodes input_file using Opus.\n");
+#if defined(HAVE_LIBFLAC)
+  printf("It can read the WAV, AIFF, FLAC, Ogg/FLAC, or raw files.\n");
+#else
+  printf("It can read the WAV, AIFF, or raw files.\n");
+#endif
   printf("\nGeneral options:\n");
   printf(" -h, --help         This help\n");
   printf(" -V, --version      Version information\n");
@@ -124,7 +129,7 @@ void usage(void)
   printf("  filename.opus     compressed file\n");
   printf("  -                 stdout\n");
   printf("\nEncoding options:\n");
-  printf(" --bitrate n.nnn    Encoding bitrate in kbit/sec (6-256 per channel)\n");
+  printf(" --bitrate n.nnn    Target bitrate in kbit/sec (6-256/channel)\n");
   printf(" --vbr              Use variable bitrate encoding (default)\n");
   printf(" --cvbr             Use constrained variable bitrate encoding\n");
   printf(" --hard-cbr         Use hard constant bitrate encoding\n");
@@ -137,6 +142,7 @@ void usage(void)
   printf(" --max-delay n      Maximum container delay in milliseconds\n");
   printf("                      (0-1000, default: 1000)\n");
   printf("\nDiagnostic options:\n");
+  printf(" --serial n         Forces a specific stream serial number\n");
   printf(" --save-range file  Saves check values for every frame to a file\n");
   printf(" --set-ctl-int x=y  Pass the encoder control x with value y (advanced)\n");
   printf("                      Preface with s: to direct the ctl to multistream s\n");
@@ -146,6 +152,75 @@ void usage(void)
   printf("                      This may be used multiple times\n");
   printf(" --artist           Author of this track\n");
   printf(" --title            Title for this track\n");
+  printf(" --album            Album or collection this track belongs to\n");
+  printf(" --date             Date for this track\n");
+  printf(" --genre            Genre for this track\n");
+  printf(" --picture          Album art for this track\n");
+  printf("                      More than one --picture option can be specified.\n");
+  printf("                      Either a FILENAME for the picture file or a more\n");
+  printf("                      complete SPECIFICATION form can be used. The\n");
+  printf("                      SPECIFICATION is a string whose parts are\n");
+  printf("                      separated by | (pipe) characters. Some parts may\n");
+  printf("                      be left empty to invoke default values. A\n");
+  printf("                      FILENAME is just shorthand for \"||||FILENAME\".\n");
+  printf("                      The format of SPECIFICATION is\n");
+  printf("\n");
+  printf("                      [TYPE]|[MIME-TYPE]|[DESCRIPTION]|[WIDTHxHEIGHT\n");
+  printf("                      xDEPTH[/COLORS]]|FILENAME\n");
+  printf("\n");
+  printf("                      TYPE is an optional number from one of:\n");
+  printf("                      0: Other\n");
+  printf("                      1: 32x32 pixel 'file icon' (PNG only)\n");
+  printf("                      2: Other file icon\n");
+  printf("                      3: Cover (front)\n");
+  printf("                      4: Cover (back)\n");
+  printf("                      5: Leaflet page\n");
+  printf("                      6: Media (e.g., label side of a CD)\n");
+  printf("                      7: Lead artist/lead performer/soloist\n");
+  printf("                      8: Artist/performer\n");
+  printf("                      9: Conductor\n");
+  printf("                      10: Band/Orchestra\n");
+  printf("                      11: Composer\n");
+  printf("                      12: Lyricist/text writer\n");
+  printf("                      13: Recording location\n");
+  printf("                      14: During recording\n");
+  printf("                      15: During performance\n");
+  printf("                      16: Movie/video screen capture\n");
+  printf("                      17: A bright colored fish\n");
+  printf("                      18: Illustration\n");
+  printf("                      19: Band/artist logotype\n");
+  printf("                      20: Publisher/studio logotype\n");
+  printf("\n");
+  printf("                      The default is 3 (front cover). There may only be\n");
+  printf("                      one picture each of type 1 and 2 in a file.\n");
+  printf("\n");
+  printf("                      MIME-TYPE is optional. If left blank, it will be\n");
+  printf("                      detected from the file. For best compatibility\n");
+  printf("                      with players, use pictures with a MIME-TYPE of\n");
+  printf("                      image/jpeg or image/png. The MIME-TYPE can also\n");
+  printf("                      be --> to mean that FILENAME is actually a URL to\n");
+  printf("                      an image, though this use is discouraged. The\n");
+  printf("                      file at the URL will not be fetched. The URL\n");
+  printf("                      itself is stored in the metadata.\n");
+  printf("\n");
+  printf("                      DESCRIPTION is optional. The default is an empty\n");
+  printf("                      string.\n");
+  printf("\n");
+  printf("                      The next part specifies the resolution and color\n");
+  printf("                      information. If the MIME-TYPE is image/jpeg,\n");
+  printf("                      image/png, or image/gif, you can usually leave\n");
+  printf("                      this empty and they can be detected from the\n");
+  printf("                      file. Otherwise, you must specify the width in\n");
+  printf("                      pixels, height in pixels, and color depth in\n");
+  printf("                      bits-per-pixel. If the image has indexed colors\n");
+  printf("                      you should also specify the number of colors\n");
+  printf("                      used. If possible, these are checked against the\n");
+  printf("                      file for accuracy.\n");
+  printf("\n");
+  printf("                      FILENAME is the path to the picture file to be\n");
+  printf("                      imported, or the URL if the MIME-TYPE is -->.\n");
+  printf(" --padding n        Extra bytes to reserve for metadata (default: 512)\n");
+  printf(" --discard-comments Don't keep metadata when transcoding\n");
   printf("\nInput options:\n");
   printf(" --raw              Raw input\n");
   printf(" --raw-bits n       Set bits/sample for raw input (default: 16)\n");
@@ -188,9 +263,9 @@ int main(int argc, char **argv)
     {"downmix-stereo",no_argument,NULL, 0},
     {"no-downmix",no_argument,NULL, 0},
     {"max-delay", required_argument, NULL, 0},
+    {"serial", required_argument, NULL, 0},
     {"save-range", required_argument, NULL, 0},
     {"set-ctl-int", required_argument, NULL, 0},
-    {"uncoupled", no_argument, NULL, 0},
     {"help", no_argument, NULL, 0},
     {"raw", no_argument, NULL, 0},
     {"raw-bits", required_argument, NULL, 0},
@@ -204,9 +279,16 @@ int main(int argc, char **argv)
     {"comment", required_argument, NULL, 0},
     {"artist", required_argument, NULL, 0},
     {"title", required_argument, NULL, 0},
+    {"album", required_argument, NULL, 0},
+    {"date", required_argument, NULL, 0},
+    {"genre", required_argument, NULL, 0},
+    {"picture", required_argument, NULL, 0},
+    {"padding", required_argument, NULL, 0},
+    {"discard-comments", no_argument, NULL, 0},
     {0, 0, 0, 0}
   };
   int i, ret;
+  int                cline_size;
   OpusMSEncoder      *st;
   const char         *opus_version;
   unsigned char      *packet;
@@ -230,9 +312,7 @@ int main(int argc, char **argv)
   int                last_segments=0;
   int                eos=0;
   OpusHeader         header;
-  int                comments_length;
-  char               ENCODER_string[64];
-  char               *comments;
+  char               ENCODER_string[1024];
   /*Counters*/
   opus_int64         nb_encoded=0;
   opus_int64         bytes_written=0;
@@ -260,13 +340,13 @@ int main(int argc, char **argv)
   int                expect_loss=0;
   int                complexity=10;
   int                downmix=0;
-  int                uncoupled=0;
   int                *opt_ctls_ctlval;
   int                opt_ctls=0;
   int                max_ogg_delay=48000; /*48kHz samples*/
+  int                seen_file_icons=0;
+  int                comment_padding=512;
+  int                serialno;
   opus_int32         lookahead=0;
-  unsigned char      mapping[256];
-  int                force_narrow=0;
 #ifdef WIN_UNICODE
    int argc_utf8;
    char **argv_utf8;
@@ -290,23 +370,30 @@ int main(int argc, char **argv)
   in_format=NULL;
   inopt.channels=chan;
   inopt.rate=coding_rate=rate;
+  /* 0 dB gain is recommended unless you know what you're doing */
+  inopt.gain=0;
   inopt.samplesize=16;
   inopt.endianness=0;
   inopt.rawmode=0;
   inopt.ignorelength=0;
+  inopt.copy_comments=1;
 
-  for(i=0;i<256;i++)mapping[i]=i;
+  start_time = time(NULL);
+  srand(((getpid()&65535)<<15)^start_time);
+  serialno=rand();
 
   opus_version=opus_get_version_string();
   /*Vendor string should just be the encoder library,
     the ENCODER comment specifies the tool used.*/
-  comment_init(&comments, &comments_length, opus_version);
-  snprintf(ENCODER_string, sizeof(ENCODER_string), "opusenc from %s %s",PACKAGE,VERSION);
-  comment_add(&comments, &comments_length, "ENCODER=", ENCODER_string);
+  comment_init(&inopt.comments, &inopt.comments_length, opus_version);
+  snprintf(ENCODER_string, sizeof(ENCODER_string), "opusenc from %s %s",PACKAGE_NAME,PACKAGE_VERSION);
+  comment_add(&inopt.comments, &inopt.comments_length, "ENCODER", ENCODER_string);
 
   /*Process command-line options*/
+  cline_size=0;
   while(1){
     int c;
+    int save_cmd=1;
     c=getopt_long(argc_utf8, argv_utf8, "hV",
                   long_options, &option_index);
     if(c==-1)
@@ -340,9 +427,11 @@ int main(int argc, char **argv)
           inopt.ignorelength=1;
         }else if(strcmp(long_options[option_index].name,"raw")==0){
           inopt.rawmode=1;
+          save_cmd=0;
         }else if(strcmp(long_options[option_index].name,"raw-bits")==0){
           inopt.rawmode=1;
           inopt.samplesize=atoi(optarg);
+          save_cmd=0;
           if(inopt.samplesize!=8&&inopt.samplesize!=16&&inopt.samplesize!=24){
             fprintf(stderr,"Invalid bit-depth: %s\n",optarg);
             fprintf(stderr,"--raw-bits must be one of 8,16, or 24\n");
@@ -351,12 +440,15 @@ int main(int argc, char **argv)
         }else if(strcmp(long_options[option_index].name,"raw-rate")==0){
           inopt.rawmode=1;
           inopt.rate=atoi(optarg);
+          save_cmd=0;
         }else if(strcmp(long_options[option_index].name,"raw-chan")==0){
           inopt.rawmode=1;
           inopt.channels=atoi(optarg);
+          save_cmd=0;
         }else if(strcmp(long_options[option_index].name,"raw-endianness")==0){
           inopt.rawmode=1;
           inopt.endianness=atoi(optarg);
+          save_cmd=0;
         }else if(strcmp(long_options[option_index].name,"downmix-mono")==0){
           downmix=1;
         }else if(strcmp(long_options[option_index].name,"downmix-stereo")==0){
@@ -397,6 +489,8 @@ int main(int argc, char **argv)
             fprintf(stderr,"max-delay 0-1000 ms.\n");
             exit(1);
           }
+        }else if(strcmp(long_options[option_index].name,"serial")==0){
+          serialno=atoi(optarg);
         }else if(strcmp(long_options[option_index].name,"set-ctl-int")==0){
           int len=strlen(optarg),target;
           char *spos,*tpos;
@@ -425,6 +519,7 @@ int main(int argc, char **argv)
           opt_ctls++;
         }else if(strcmp(long_options[option_index].name,"save-range")==0){
           frange=fopen_utf8(optarg,"w");
+          save_cmd=0;
           if(frange==NULL){
             perror(optarg);
             fprintf(stderr,"Could not open save-range file: %s\n",optarg);
@@ -432,20 +527,50 @@ int main(int argc, char **argv)
             exit(1);
           }
           range_file=optarg;
-        }else if(strcmp(long_options[option_index].name,"uncoupled")==0){
-          uncoupled=1;
         }else if(strcmp(long_options[option_index].name,"comment")==0){
+          save_cmd=0;
           if(!strchr(optarg,'=')){
             fprintf(stderr, "Invalid comment: %s\n", optarg);
             fprintf(stderr, "Comments must be of the form name=value\n");
             exit(1);
           }
-          comment_add(&comments, &comments_length, NULL, optarg);
+          comment_add(&inopt.comments, &inopt.comments_length, NULL, optarg);
         }else if(strcmp(long_options[option_index].name,"artist")==0){
-          comment_add(&comments, &comments_length, "artist=", optarg);
+          save_cmd=0;
+          comment_add(&inopt.comments, &inopt.comments_length, "artist", optarg);
         } else if(strcmp(long_options[option_index].name,"title")==0){
-          comment_add(&comments, &comments_length, "title=", optarg);
+          save_cmd=0;
+          comment_add(&inopt.comments, &inopt.comments_length, "title", optarg);
+        } else if(strcmp(long_options[option_index].name,"album")==0){
+          save_cmd=0;
+          comment_add(&inopt.comments, &inopt.comments_length, "album", optarg);
+        } else if(strcmp(long_options[option_index].name,"date")==0){
+          save_cmd=0;
+          comment_add(&inopt.comments, &inopt.comments_length, "date", optarg);
+        } else if(strcmp(long_options[option_index].name,"genre")==0){
+          save_cmd=0;
+          comment_add(&inopt.comments, &inopt.comments_length, "genre", optarg);
+        } else if(strcmp(long_options[option_index].name,"picture")==0){
+          const char *error_message;
+          char       *picture_data;
+          save_cmd=0;
+          picture_data=parse_picture_specification(optarg,&error_message,
+                                                   &seen_file_icons);
+          if(picture_data==NULL){
+            fprintf(stderr,"Error parsing picture option: %s\n",error_message);
+            exit(1);
+          }
+          comment_add(&inopt.comments,&inopt.comments_length,
+                      "METADATA_BLOCK_PICTURE",picture_data);
+          free(picture_data);
+        } else if(strcmp(long_options[option_index].name,"padding")==0){
+          comment_padding=atoi(optarg);
+        } else if(strcmp(long_options[option_index].name,"discard-comments")==0){
+          inopt.copy_comments=0;
         }
+        /*Commands whos arguments would leak file paths or just end up as metadata
+           should have save_cmd=0; to prevent them from being saved in the
+           command-line tag.*/
         break;
       case 'h':
         usage();
@@ -460,6 +585,22 @@ int main(int argc, char **argv)
         exit(1);
         break;
     }
+    if(save_cmd && cline_size<(int)sizeof(ENCODER_string)){
+      ret=snprintf(&ENCODER_string[cline_size], sizeof(ENCODER_string)-cline_size, "%s--%s",cline_size==0?"":" ",long_options[option_index].name);
+      if(ret<0||ret>=((int)sizeof(ENCODER_string)-cline_size)){
+        cline_size=sizeof(ENCODER_string);
+      } else {
+        cline_size+=ret;
+        if(optarg){
+          ret=snprintf(&ENCODER_string[cline_size], sizeof(ENCODER_string)-cline_size, " %s",optarg);
+          if(ret<0||ret>=((int)sizeof(ENCODER_string)-cline_size)){
+            cline_size=sizeof(ENCODER_string);
+          } else {
+            cline_size+=ret;
+          }
+        }
+      }
+    }
   }
   if(argc_utf8-optind!=2){
     usage();
@@ -468,6 +609,8 @@ int main(int argc, char **argv)
   inFile=argv_utf8[optind];
   outFile=argv_utf8[optind+1];
 
+  if(cline_size>0)comment_add(&inopt.comments, &inopt.comments_length, "ENCODER_OPTIONS", ENCODER_string);
+
   if(strcmp(inFile, "-")==0){
 #if defined WIN32 || defined _WIN32
     _setmode(_fileno(stdin), _O_BINARY);
@@ -493,8 +636,8 @@ int main(int argc, char **argv)
     exit(1);
   }
 
-  if(downmix==0&&inopt.channels>2&&bitrate>0&&bitrate<(32000*inopt.channels)){
-    if(!quiet)fprintf(stderr,"Notice: Surround bitrate less than 32kbit/sec/channel, downmixing.\n");
+  if(downmix==0&&inopt.channels>2&&bitrate>0&&bitrate<(16000*inopt.channels)){
+    if(!quiet)fprintf(stderr,"Notice: Surround bitrate less than 16kbit/sec/channel, downmixing.\n");
     downmix=inopt.channels>8?1:2;
   }
 
@@ -527,30 +670,19 @@ int main(int argc, char **argv)
 
   /*OggOpus headers*/ /*FIXME: broke forcemono*/
   header.channels=chan;
-  header.nb_streams=header.channels;
-  header.nb_coupled=0;
-  if(header.channels<=8&&!uncoupled){
-    static const unsigned char opusenc_streams[8][10]={
-      /*Coupled, NB_bitmap, mapping...*/
-      /*1*/ {0,   0, 0},
-      /*2*/ {1,   0, 0,1},
-      /*3*/ {1,   0, 0,2,1},
-      /*4*/ {2,   0, 0,1,2,3},
-      /*5*/ {2,   0, 0,4,1,2,3},
-      /*6*/ {2,1<<3, 0,4,1,2,3,5},
-      /*7*/ {2,1<<4, 0,4,1,2,3,5,6},
-      /*6*/ {3,1<<4, 0,6,1,2,3,4,5,7}
-    };
-    for(i=0;i<header.channels;i++)mapping[i]=opusenc_streams[header.channels-1][i+2];
-    force_narrow=opusenc_streams[header.channels-1][1];
-    header.nb_coupled=opusenc_streams[header.channels-1][0];
-    header.nb_streams=header.channels-header.nb_coupled;
-  }
-  header.channel_mapping=header.channels>8?255:header.nb_streams>1;
-  if(header.channel_mapping>0)for(i=0;i<header.channels;i++)header.stream_map[i]=mapping[i];
-  /* 0 dB gain is the recommended unless you know what you're doing */
-  header.gain=0;
+  header.channel_mapping=header.channels>8?255:chan>2;
   header.input_sample_rate=rate;
+  header.gain=inopt.gain;
+
+  /*Initialize OPUS encoder*/
+  /*Framesizes <10ms can only use the MDCT modes, so we switch on RESTRICTED_LOWDELAY
+    to save the extra 2.5ms of codec lookahead when we'll be using only small frames.*/
+  st=opus_multistream_surround_encoder_create(coding_rate, chan, header.channel_mapping, &header.nb_streams, &header.nb_coupled,
+     header.stream_map, frame_size<480/(48000/coding_rate)?OPUS_APPLICATION_RESTRICTED_LOWDELAY:OPUS_APPLICATION_AUDIO, &ret);
+  if(ret!=OPUS_OK){
+    fprintf(stderr, "Error cannot create encoder: %s\n", opus_strerror(ret));
+    exit(1);
+  }
 
   min_bytes=max_frame_bytes=(1275*3+7)*header.nb_streams;
   packet=malloc(sizeof(unsigned char)*max_frame_bytes);
@@ -559,30 +691,6 @@ int main(int argc, char **argv)
     exit(1);
   }
 
-  /*Initialize OPUS encoder*/
-  /*Framesizes <10ms can only use the MDCT modes, so we switch on RESTRICTED_LOWDELAY
-    to save the extra 2.5ms of codec lookahead when we'll be using only small frames.*/
-  st=opus_multistream_encoder_create(coding_rate, chan, header.nb_streams, header.nb_coupled,
-     mapping, frame_size<480/(48000/coding_rate)?OPUS_APPLICATION_RESTRICTED_LOWDELAY:OPUS_APPLICATION_AUDIO, &ret);
-  if(ret!=OPUS_OK){
-    fprintf(stderr, "Error cannot create encoder: %s\n", opus_strerror(ret));
-    exit(1);
-  }
-
-  if(force_narrow!=0){
-    for(i=0;i<header.nb_streams;i++){
-      if(force_narrow&(1<<i)){
-        OpusEncoder *oe;
-        opus_multistream_encoder_ctl(st,OPUS_MULTISTREAM_GET_ENCODER_STATE(i,&oe));
-        ret=opus_encoder_ctl(oe, OPUS_SET_MAX_BANDWIDTH(OPUS_BANDWIDTH_NARROWBAND));
-        if(ret!=OPUS_OK){
-          fprintf(stderr,"Error OPUS_SET_MAX_BANDWIDTH on stream %d returned: %s\n",i,opus_strerror(ret));
-          exit(1);
-        }
-      }
-    }
-  }
-
   if(bitrate<0){
     /*Lower default rate for sampling rates [8000-44100) by a factor of (rate+16k)/(64k)*/
     bitrate=((64000*header.nb_streams+32000*header.nb_coupled)*
@@ -710,9 +818,7 @@ int main(int argc, char **argv)
   }
 
   /*Initialize Ogg stream struct*/
-  start_time = time(NULL);
-  srand(((getpid()&65535)<<15)^start_time);
-  if(ogg_stream_init(&os, rand())==-1){
+  if(ogg_stream_init(&os, serialno)==-1){
     fprintf(stderr,"Error: stream init failed\n");
     exit(1);
   }
@@ -740,8 +846,9 @@ int main(int argc, char **argv)
       pages_out++;
     }
 
-    op.packet=(unsigned char *)comments;
-    op.bytes=comments_length;
+    comment_pad(&inopt.comments, &inopt.comments_length, comment_padding);
+    op.packet=(unsigned char *)inopt.comments;
+    op.bytes=inopt.comments_length;
     op.b_o_s=0;
     op.e_o_s=0;
     op.granulepos=0;
@@ -761,7 +868,7 @@ int main(int argc, char **argv)
     pages_out++;
   }
 
-  free(comments);
+  free(inopt.comments);
 
   input=malloc(sizeof(float)*frame_size*chan);
   if(input==NULL){
@@ -937,20 +1044,20 @@ int main(int argc, char **argv)
     double wall_time=(stop_time-start_time)+1e-6;
     fprintf(stderr,"Encoding complete                                    \n");
     fprintf(stderr,"-----------------------------------------------------\n");
-    fprintf(stderr,"    Encoded:");
+    fprintf(stderr,"       Encoded:");
     print_time(coded_seconds);
-    fprintf(stderr,"\n    Runtime:");
+    fprintf(stderr,"\n       Runtime:");
     print_time(wall_time);
-    fprintf(stderr,"\n             (%0.4gx realtime)\n",coded_seconds/wall_time);
-    fprintf(stderr,"      Wrote: %" I64FORMAT " bytes, %d packets, %" I64FORMAT " pages\n",bytes_written,id+1,pages_out);
-    fprintf(stderr,"    Bitrate: %0.6gkbit/s (without overhead)\n",
+    fprintf(stderr,"\n                (%0.4gx realtime)\n",coded_seconds/wall_time);
+    fprintf(stderr,"         Wrote: %" I64FORMAT " bytes, %d packets, %" I64FORMAT " pages\n",bytes_written,id+1,pages_out);
+    fprintf(stderr,"       Bitrate: %0.6gkbit/s (without overhead)\n",
             total_bytes*8.0/(coded_seconds)/1000.0);
-    fprintf(stderr," Rate range: %0.6gkbit/s to %0.6gkbit/s\n             (%d to %d bytes per packet)\n",
+    fprintf(stderr," Instant rates: %0.6gkbit/s to %0.6gkbit/s\n                (%d to %d bytes per packet)\n",
             min_bytes*8*((double)coding_rate/frame_size/1000.),
             peak_bytes*8*((double)coding_rate/frame_size/1000.),min_bytes,peak_bytes);
-    fprintf(stderr,"   Overhead: %0.3g%% (container+metadata)\n",(bytes_written-total_bytes)/(double)bytes_written*100.);
+    fprintf(stderr,"      Overhead: %0.3g%% (container+metadata)\n",(bytes_written-total_bytes)/(double)bytes_written*100.);
 #ifdef OLD_LIBOGG
-    if(max_ogg_delay>(frame_size*(48000/coding_rate)*4))fprintf(stderr,"   (use libogg 1.3 or later for lower overhead)\n");
+    if(max_ogg_delay>(frame_size*(48000/coding_rate)*4))fprintf(stderr,"    (use libogg 1.3 or later for lower overhead)\n");
 #endif
     fprintf(stderr,"\n");
   }
@@ -1021,12 +1128,12 @@ static void comment_init(char **comments, int* length, const char *vendor_string
   *comments=p;
 }
 
-static void comment_add(char **comments, int* length, char *tag, char *val)
+void comment_add(char **comments, int* length, char *tag, char *val)
 {
   char* p=*comments;
   int vendor_length=readint(p, 8);
   int user_comment_list_length=readint(p, 8+4+vendor_length);
-  int tag_len=(tag?strlen(tag):0);
+  int tag_len=(tag?strlen(tag)+1:0);
   int val_len=strlen(val);
   int len=(*length)+4+tag_len+val_len;
 
@@ -1037,11 +1144,34 @@ static void comment_add(char **comments, int* length, char *tag, char *val)
   }
 
   writeint(p, *length, tag_len+val_len);      /* length of comment */
-  if(tag) memcpy(p+*length+4, tag, tag_len);  /* comment */
+  if(tag){
+    memcpy(p+*length+4, tag, tag_len);        /* comment tag */
+    (p+*length+4)[tag_len-1] = '=';           /* separator */
+  }
   memcpy(p+*length+4+tag_len, val, val_len);  /* comment */
   writeint(p, 8+4+vendor_length, user_comment_list_length+1);
   *comments=p;
   *length=len;
 }
+
+static void comment_pad(char **comments, int* length, int amount)
+{
+  if(amount>0){
+    int i;
+    int newlen;
+    char* p=*comments;
+    /*Make sure there is at least amount worth of padding free, and
+       round up to the maximum that fits in the current ogg segments.*/
+    newlen=(*length+amount+255)/255*255-1;
+    p=realloc(p,newlen);
+    if(p==NULL){
+      fprintf(stderr,"realloc failed in comment_pad()\n");
+      exit(1);
+    }
+    for(i=*length;i<newlen;i++)p[i]=0;
+    *comments=p;
+    *length=newlen;
+  }
+}
 #undef readint
 #undef writeint
